
class Scrollbars {
    /**
     * The scrollbar DOM element.
     *
     * @type {DOM}
     */
    scrollbar: HTMLElement;

    /**
     * Create a new instance.
     *
     * @param  {string} selector
     */
    constructor(selector: string) {
        this.scrollbar = <HTMLElement>document.querySelector(selector);
        this.height(this.scrollbar);
        this.onScroll(this.scrollbar);
        this.onClick(this.scrollbar);

        window.addEventListener('resize', () => this.height(this.scrollbar));
    }

    /**
     * Determine the height of the scrollbar.
     *
     * @param  {HTMLElement} element
     * @return {void}
     */
    position(element: HTMLElement): void {
        let ratio = (element.scrollTop / element.scrollHeight);
        let data_scroll = Math.round(ratio * 1000);

        element.dataset['scroll'] = data_scroll.toString();
    }

    /**
     * Adjust the scroll bar height.
     *
     * @param {HTMLElement} element
     * @return {void}
     */
    height(element: HTMLElement): void {
        let ratio = element.clientHeight / element.scrollHeight;
        let height = ratio * 100;
        let data_height = Math.floor(height);

        element.dataset['scrollBar'] = data_height.toString();
    }

    /**
     * Set the right margin of the scrolling element.
     *
     * @param {HTMLElement} element
     * @return {void}
     */
    margin(element: HTMLElement): void {
        element.style.marginRight = '-15px';
    }

    /**
     * On scroll event handler.
     *
     * @param {HTMLElement} element
     * @return {void}
     */
    onScroll(element: HTMLElement): void {
        this.scrollbar.addEventListener('scroll', (event: MouseEvent) => {
            this.position(<HTMLElement>event.target);
            this.height(<HTMLElement>event.target);
        });
    }

    /**
     * On click event handler.
     *
     * @param  {HTMLElement} element
     * @return {void}
     */
    onClick(element: HTMLElement): void {
        element.addEventListener('click', (e: MouseEvent) => {
            if (e.target !== element) return;

            if (e.target['offsetWidth'] - e.x < 30) {
                let click_percentage = Math.round((e.clientY / this.scrollbar.clientHeight) * 100);
                let scroll_top = (this.scrollbar.scrollHeight * click_percentage) / 100;

                this.scrollbar.scrollTop = scroll_top;
            }
        });
    }

    onDrag(element: HTMLElement): void {
        element.addEventListener('click', (e: MouseEvent) => {
            if (e.target !== element) return;

            if (e.target['offsetWidth'] - e.x < 30) {

            }
        });

    }
}

var scrollbars = new Scrollbars('.scrollbars');

export = Scrollbars;
